import { Component, createEffect } from 'solid-js';

import style from './index.module.css';
import Tree from '../tree/Tree';
import getUri from '../../lib/ids';
import { createCachedSignal } from '../../workers/cached-signals';
import { FormControlLabel, Radio, RadioGroup } from '@suid/material';
import { getDrawingSelections } from '../../db/drawing-selections';
import dispatch from '../../workers/dispatcher-main';
import { Feature } from 'ol';
import uri from '../../lib/ids';
import { currentGpxId } from '../gpx-dialog';

const id = getUri('drawingSelections', undefined);

type CompleteSelectionType = 'ALL' | 'NONE' | 'CURRENT';

type RestrictedSelectionType = 'ALL' | 'NONE';

type DrawingSelections = {
  gpxSelection: CompleteSelectionType;
  trkSelection: RestrictedSelectionType;
  rteSelection: RestrictedSelectionType;
  wptSelection: RestrictedSelectionType;
};

const defaultDrawingSelections: DrawingSelections = {
  gpxSelection: 'ALL',
  trkSelection: 'ALL',
  rteSelection: 'ALL',
  wptSelection: 'ALL',
};

const drawingSelections = createCachedSignal({
  id,
  method: 'getDrawingSelections',
  defaultDrawingSelections,
}) as () => DrawingSelections;

createEffect(() => {
  console.log({
    caller: 'DrawingSelections',
    drawingSelections: drawingSelections(),
  });
});

const AllNoneCurrent: Component<{}> = (props) => {
  return (
    <>
      <FormControlLabel
        value='ALL'
        control={<Radio />}
        label='All'
      ></FormControlLabel>
      <FormControlLabel
        value='NONE'
        control={<Radio />}
        label='None'
      ></FormControlLabel>
      <FormControlLabel
        value='CURRENT'
        control={<Radio />}
        label='Current'
      ></FormControlLabel>
    </>
  );
};

const AllNone: Component<{}> = (props) => {
  return (
    <>
      <FormControlLabel
        value='ALL'
        control={<Radio />}
        label='All'
      ></FormControlLabel>
      <FormControlLabel
        value='NONE'
        control={<Radio />}
        label='None'
      ></FormControlLabel>
    </>
  );
};

const gpxSelection = () => drawingSelections().gpxSelection;

const gpxMustBeDrawn = (feature: Feature) => {
  const id = feature.getId() as String;
  if (gpxSelection() === 'ALL') {
    return true;
  }
  if (gpxSelection() === 'CURRENT') {
    return id.startsWith(currentGpxId());
  }
  return false;
};

const typeMustBeDrawn = (feature: Feature) => {
  const type = feature.get('type') as String;
  if (type.startsWith('trk')) {
    return drawingSelections().trkSelection !== 'NONE';
  }
  if (type === 'wpt') {
    return drawingSelections().wptSelection !== 'NONE';
  }
  if (type === 'rte') {
    return drawingSelections().rteSelection !== 'NONE';
  }
  return true;
};

export const mustBeDrawn = (feature: Feature) => {
  console.log({
    caller: 'DrawingSelector / mustBeDrawn',
    feature,
  });
  return gpxMustBeDrawn(feature) && typeMustBeDrawn(feature);
};

const handleGpxSelectionChange = (ev: any) => {
  const value = ev.target.value;
  console.log({
    caller: 'DrawingSelector / handleGpxSelectionChange',
    ev,
    value,
  });
  const newDrawingSelections = drawingSelections();
  newDrawingSelections.gpxSelection = value;
  dispatch({
    action: 'putDrawingSelections',
    params: { id, drawingSelections: newDrawingSelections },
  });
};

const handleTrkSelectionChange = (ev: any) => {
  const value = ev.target.value;
  console.log({
    caller: 'DrawingSelector / handleTrkSelectionChange',
    ev,
    value,
  });
  const newDrawingSelections = drawingSelections();
  newDrawingSelections.trkSelection = value;
  dispatch({
    action: 'putDrawingSelections',
    params: { id, drawingSelections: newDrawingSelections },
  });
};

const handleRteSelectionChange = (ev: any) => {
  const value = ev.target.value;
  console.log({
    caller: 'DrawingSelector / handleRteSelectionChange',
    ev,
    value,
  });
  const newDrawingSelections = drawingSelections();
  newDrawingSelections.rteSelection = value;
  dispatch({
    action: 'putDrawingSelections',
    params: { id, drawingSelections: newDrawingSelections },
  });
};

const handleWptSelectionChange = (ev: any) => {
  const value = ev.target.value;
  console.log({
    caller: 'DrawingSelector / handleWptSelectionChange',
    ev,
    value,
  });
  const newDrawingSelections = drawingSelections();
  newDrawingSelections.wptSelection = value;
  dispatch({
    action: 'putDrawingSelections',
    params: { id, drawingSelections: newDrawingSelections },
  });
};

const DrawingSelector: Component<{}> = (props) => {
  return (
    <Tree
      title='Show'
      content={
        <>
          <div>Trips</div>
          <RadioGroup
            defaultValue={drawingSelections()?.gpxSelection}
            onChange={handleGpxSelectionChange}
          >
            <AllNoneCurrent />
          </RadioGroup>
          <div>Tracks</div>
          <RadioGroup
            defaultValue={drawingSelections()?.trkSelection}
            onChange={handleTrkSelectionChange}
          >
            <AllNone />
          </RadioGroup>
          <div>Routes</div>
          <RadioGroup
            defaultValue={drawingSelections()?.rteSelection}
            onChange={handleRteSelectionChange}
          >
            <AllNone />
          </RadioGroup>{' '}
          <div>Waypoints</div>
          <RadioGroup
            defaultValue={drawingSelections()?.wptSelection}
            onChange={handleWptSelectionChange}
          >
            <AllNone />
          </RadioGroup>
        </>
      }
      subTree={undefined}
    />
  );
};

export default DrawingSelector;
