import { Component, createEffect, For, onCleanup, Suspense } from 'solid-js';

import OlMap from 'ol/Map';

import Trk from '../trk';
import VectorSource from 'ol/source/Vector';
import VectorLayer from 'ol/layer/Vector';

import style from './styles';
import Wpt from '../wpt';
import Rte from '../rte';
import {
  createCachedSignal,
  destroyCachedSignal,
} from '../../workers/cached-signals';
import { getTagsFromObj } from '../../lib/tags/tags';

interface Props {
  gpxId: string;
  map: () => OlMap | null;
}

export const Gpx: Component<Props> = ({ map, gpxId }) => {
  const params = {
    id: gpxId,
    method: 'getGpx',
  };
  const gpx = createCachedSignal(params);

  onCleanup(() => {
    console.log({
      caller: 'Gpx / onCleanup',
      gpxId,
      gpx: gpx(),
      params,
    });

    destroyCachedSignal(params);
  });

  const vectorSources = [new VectorSource(), new VectorSource()];
  const vectorLayers = [
    new VectorLayer({
      source: vectorSources[0],
      style,
      declutter: true,
      zIndex: 1,
    }),
    new VectorLayer({
      source: vectorSources[1],
      style,
      declutter: false,
      zIndex: 2,
    }),
  ];
  createEffect(() => {
    map()?.addLayer(vectorLayers[0]);
    map()?.addLayer(vectorLayers[1]);
  });

  createEffect(() => {
    console.log({ caller: 'Gpx', map: map(), gpxId, gpx: gpx(), vectorLayers });
  });

  return (
    <>
      <For each={gpx() ? gpx().trk || [] : []}>
        {(trkId: string) => {
          console.log({ caller: 'Gpx / loop', trkId });
          return (
            <Suspense>
              <Trk
                vectorSources={vectorSources}
                trkId={trkId}
                context={{ gpx, gpxId }}
                tags={getTagsFromObj(gpx())}
              />
            </Suspense>
          );
        }}
      </For>
      <For each={gpx() ? gpx().rte || [] : []}>
        {(rteId: string) => {
          console.log({ caller: 'Gpx / loop', rteId });
          return (
            <Suspense>
              <Rte
                vectorSources={vectorSources}
                rteId={rteId}
                context={{ gpx, gpxId }}
              />
            </Suspense>
          );
        }}
      </For>
      <For each={gpx() ? gpx().wpt || [] : []}>
        {(wptId: string) => {
          console.log({ caller: 'Gpx / loop', wptId });
          return (
            <Suspense>
              <Wpt
                vectorSources={vectorSources}
                wptId={wptId}
                context={{ gpx, gpxId }}
              />
            </Suspense>
          );
        }}
      </For>
    </>
  );
};

export default Gpx;
