import { useI18n } from '@solid-primitives/i18n';
import {
  Typography,
  Card,
  CardContent,
  CardActions,
  Button,
  Box,
  LinearProgress,
  CircularProgress,
} from '@suid/material';
import { Component, createEffect, createSignal, Show } from 'solid-js';
import { findStartTime } from '../../lib/gpx';
import GPX from '../../lib/gpx-parser-builder/src/gpx';
import dispatch from '../../workers/dispatcher-main';
import GpxChooser from '../gpx-chooser';
import { currentGpxId } from '../gpx-dialog';
import {
  Filesystem as CapacitorFileSystem,
  Encoding,
} from '@capacitor/filesystem';
import piexif from 'piexifjs';
import { getDate, getGps } from '../../lib/exif';
import { downsize } from '../../lib/image';
import { Coordinate } from 'ol/coordinate';
import WptEditDialog, { Category } from '../wpt/WptEditDialog';

interface Props {
  file: File;
}

interface StatsAndGpx {
  gpx: Gpx;
  stats: any;
}

interface Picture {
  id: string;
  date: string;
  gpx: { Latitude: number; Longitude: number; Altitude: number };
  thumbnailUrl: string;
}

const analyzeGpx = (gpx: Gpx | undefined) => {
  if (gpx === undefined) {
    return {};
  }
  const stats = {
    creator: gpx.$.creator,
    nbWpts: gpx.wpt ? gpx.wpt?.length : 0,
    nbRtes: gpx.rte ? gpx.rte?.length : 0,
    nbTrks: gpx.trk ? gpx.trk?.length : 0,
    trkMaybeRte:
      gpx.trk &&
      gpx.trk[0].trkseg &&
      gpx.trk[0].trkseg[0] &&
      gpx.trk[0].trkseg[0].trkpt &&
      !gpx.trk[0].trkseg[0].trkpt[0].time,
    startTime: findStartTime(gpx),
  };

  return stats;
};

const ImportSingleFile: Component<Props> = ({ file: file }) => {
  const [t] = useI18n();
  const [statsAndGpx, setStatsAndGpx] = createSignal<StatsAndGpx>();
  const [picture, setPicture] = createSignal<Picture>();
  const [state, setState] = createSignal('init');
  const [gpxId, setGpxId] = createSignal<string>('new');

  const [open, setOpen] = createSignal<boolean>(false);
  const [coordinate, setCoordinate] = createSignal<Coordinate>();
  const [initialWpt, setInitialWpt] = createSignal({
    $: { lat: 0, lon: 0 },
    ele: undefined,
    time: undefined,
    magvar: undefined,
    geoidheight: undefined,
    name: undefined,
    cmt: undefined,
    desc: undefined,
    src: undefined,
    link: undefined,
    sym: undefined,
    type: undefined,
    fix: undefined,
    sat: undefined,
    hdop: undefined,
    vdop: undefined,
    pdop: undefined,
    ageofdgpsdata: undefined,
    dgpsid: undefined,
    extensions: {
      address: undefined,
      startTime: undefined,
      endTime: undefined,
      category: Category.PICTURE,
      subCategory: undefined,
      pictureId: '',
      thumbnailUrl: '',
    },
  });

  createEffect(() => {
    setGpxId(currentGpxId());
  });

  const parseGpx = (content: string) => {
    const gpx = GPX.parse(content);
    console.log({
      caller: 'ImportSingleFile / JSON',
      gpxFile: file,
      gpx,
      gpxId: gpxId(),
    });
    setStatsAndGpx({ gpx, stats: analyzeGpx(gpx) });
  };

  if (typeof file === 'string') {
    CapacitorFileSystem.requestPermissions().then((permissionStatus) => {
      console.log({
        caller: 'ImportSingleFile / content',
        permissionStatus,
      });
    });
    CapacitorFileSystem.readFile({
      path: file,
      encoding: Encoding.UTF8,
    }).then((content) => {
      console.log({
        caller: 'ImportSingleFile / content',
        file,
        content,
      });
      parseGpx(content.data);
    });
  } else {
    const reader = new FileReader();
    if (file.type === 'image/jpeg') {
      reader.readAsDataURL(file);
      reader.addEventListener(
        'load',
        async () => {
          const imageUrl = reader.result;
          const exifObj = piexif.load(imageUrl);
          console.log({
            caller: 'ImportSingleFile / Jpeg',
            file,
            type: file.type,
            result: reader.result,
            exifObj,
          });
          const id = exifObj.Exif[piexif.ExifIFD.ImageUniqueID];
          const gps = getGps(exifObj);
          const date = getDate(exifObj);
          const [downsizedUrl, bigThumbnailUrl, thumbnailUrl] = await downsize(
            reader.result,
            [1024, 512, 128]
          );
          console.log({
            caller: 'ImportSingleFile / Jpeg',
            file,
            id,
            gps,
            date,
            type: file.type,
            result: reader.result,
            exifObj,
          });
          setPicture({
            id,
            gps,
            date,
            thumbnailUrl,
            bigThumbnailUrl,
            downsizedUrl,
          });
          setCoordinate([gps.longitude, gps.latitude]);
          setInitialWpt({
            ...initialWpt(),
            extensions: {
              ...initialWpt().extensions,
              category: Category.PICTURE,
              startTime: date.toISOString(),
              endTime: date.toISOString(),
              pictureId: id,
              thumbnailUrl: thumbnailUrl,
            },
          });
        },
        false
      );
    } else {
      // GPX
      reader.readAsText(file);
      reader.addEventListener(
        'load',
        async () => {
          // this will then display a text gpxfile
          console.log({
            caller: 'ImportSingleFile / XML',
            file,
            type: file.type,
            result: reader.result,
          });
          parseGpx(reader.result);
        },
        false
      );
    }
  }
  const doImport = async (forceImportAsRoutes: boolean = false) => {
    setState('importing');
    //       console.log({ caller: 'GpxImport / JSON', file, gpx });
    //       if (gpx) {
    //         const startTime = new Date(findStartTime(gpx)!);
    await dispatch({
      action: 'pruneAndSaveImportedGpx',
      params: {
        id: gpxId(),
        forceImportAsRoutes,
        gpx: statsAndGpx()?.gpx,
        tech:
          typeof file === 'string'
            ? {
                importDate: new Date().toISOString(),
                uri: file,
              }
            : {
                lastModified: new Date(file.lastModified).toISOString(),
                importDate: new Date().toISOString(),
                name: file.name,
                size: file.size,
                type: file.type,
              },
      },
    });
    console.log({
      caller: 'GpxImport / JSON / done',
      gpxFile: file,
      gpx: statsAndGpx()?.gpx,
    });
    setState('done');

    //       } else {
    //         console.error({
    //           message: "can't parse GPX file",
    //           file,
    //           xml: fileReader.result,
    //         });
    //       }
    //       // TODO: error handling
    //     },
    //     false
    //   );
    // }
  };

  const [selectedTrkTransform, setSelectedTrkTransform] = createSignal(
    'importNonTimedTrksAsRtes'
  );

  const handleTrkTransformChange = (event: any) => {
    setSelectedTrkTransform(event.target.value);
  };

  const controlTrkTransformProps = (item: string) => ({
    checked: selectedTrkTransform() === item,
    onChange: handleTrkTransformChange,
    value: item,
    name: 'trkTransform',
    label: t(item),
    inputProps: { 'aria-label': item },
  });

  return (
    <Card raised>
      <CardContent>
        <Typography variant='h6'>
          {t('file')}
          {file.name}
        </Typography>
      </CardContent>
      <CardContent>
        <Show when={statsAndGpx() !== undefined}>
          <Typography variant='body1'>
            {t('gpxStats', statsAndGpx().stats)}{' '}
            {statsAndGpx()?.stats.startTime
              ? t('gpxStartTime', statsAndGpx().stats)
              : t('gpxNoStartTime')}
          </Typography>
          <Show when={statsAndGpx().stats.trkMaybeRte}>
            <Typography variant='body1'>{t('trkMaybeRte')}</Typography>
          </Show>
          <Show when={state() === 'importing'}>
            <LinearProgress />
          </Show>
          <CardActions>
            <GpxChooser
              gpxId={gpxId}
              setGpxId={setGpxId}
              disabled={state() != 'init'}
            />
            <Button
              variant='contained'
              disabled={state() != 'init'}
              onClick={() => doImport(false)}
            >
              {t('import')}
            </Button>
            <Button
              variant='contained'
              disabled={state() != 'init'}
              onClick={() => doImport(true)}
            >
              {t('importAllTrksAsRtes')}
            </Button>
          </CardActions>
        </Show>
        <Show when={picture() !== undefined}>
          <img src={picture()?.bigThumbnailUrl} />
          <p>Id: {picture()?.id}</p>
          <p>Date: {picture()?.date.toString()}</p>
          <p>
            Coordinates: {picture()?.gps.latitude}, {picture()?.gps.longitude}
          </p>
          <CardActions>
            <Button
              variant='contained'
              disabled={state() != 'init'}
              onClick={() => setOpen(true)}
            >
              {t('import')}
            </Button>
          </CardActions>
          <Show when={open()}>
            <WptEditDialog
              open={open}
              closeHandler={() => {
                setOpen(false);
                setState('imported');
              }}
              coordinate={coordinate}
              initialWpt={initialWpt}
              imageUrl={picture()?.downsizedUrl}
            />
          </Show>
        </Show>
      </CardContent>
    </Card>
  );
};

export default ImportSingleFile;
