import {
  ListItem,
  ListItemButton,
  ListItemIcon,
  ListItemText,
} from '@suid/material';
import {
  Component,
  createEffect,
  createSignal,
  For,
  onMount,
  Show,
} from 'solid-js';
import SystemSecurityUpdateWarningIcon from '@suid/icons-material/SystemSecurityUpdateWarning';
import Dialog from '../dialog';
import dispatch from '../../workers/dispatcher-main';
import { sleep } from '../../lib/async-wait';
import style from './SystemInfo.module.css';

const [error, setError] = createSignal<any[]>([]);

export const reportError = (err: any) => {
  setError([...error(), err]);
};

interface Props {}

const SystemInfo: Component<Props> = (props) => {
  const [open, setOpen] = createSignal(false);
  const [settings, setSettings] = createSignal<any>();
  const [account, setAccount] = createSignal<any>();

  createEffect(() => {
    if (error().length > 0 && !open()) {
      openDialog();
    }
  });

  const selfTest = async () => {
    try {
      const pong = await dispatch({
        action: 'ping',
        params: { what: 'ping' },
      });
      if (pong !== 'ping pong') {
        throw new Error(
          `web worker test error, received "${pong}" instead of expected "ping pong"`
        );
      }
    } catch (err) {
      reportError(err);
    }
  };

  selfTest();

  const openDialog = async () => {
    setOpen(true);
    try {
      const sets: any = await dispatch({
        action: 'getSettings',
      });
      setSettings(sets);
      const acc: any = await dispatch({
        action: 'getAccount',
        params: { id: settings().currentAccountId },
      });
      setAccount(acc);
    } catch (err) {
      reportError(err);
    }
  };

  let content: any;

  onMount(() => {
    if (!!content) {
      content.onclick = function () {
        document.execCommand('copy');
      };

      content.addEventListener('copy', function (event: any) {
        event.preventDefault();
        if (event.clipboardData) {
          event.clipboardData.setData('text/plain', content.textContent);
          console.log(event.clipboardData.getData('text'));
        }
      });
    }
  });

  const clickHandler = async (event: any) => {
    const range = document.createRange();
    range.selectNode(content);
    window.getSelection()?.removeAllRanges();
    window.getSelection()?.addRange(range);
    document.execCommand('copy');
    await sleep(300);
    window.getSelection()?.removeAllRanges();
    console.log({ caller: 'SystemInfo / clickHandler', event, range });
  };

  return (
    <>
      <ListItem>
        <ListItemButton onClick={openDialog}>
          <ListItemIcon>
            <SystemSecurityUpdateWarningIcon />
          </ListItemIcon>
          <ListItemText>System Info</ListItemText>
        </ListItemButton>
      </ListItem>
      <Show when={open()}>
        <Dialog
          open={open()}
          closeHandler={() => setOpen(false)}
          title='System info'
        >
          <div onclick={clickHandler}>
            <div class={style.clickPrompt}>Click to copy to clipboard</div>
            <div ref={content}>
              <p>Platform: {window.Capacitor.platform}</p>
              <p>User agent: {navigator.userAgent}</p>
              <p>App release: {import.meta.env.VITE_GIT_COMMIT_HASH}</p>
              <p>Account id: {settings()?.currentAccountId}</p>
              <p>Account name: {account()?.name}</p>
              <p>Remote DB user: {account()?.remoteDbUser}</p>
              <p>Remote DB server: {account()?.remoteDbServer}</p>
            </div>
            <Show when={error().length > 0}>
              <div>
                <p>Errors:</p>
                <ul>
                  <For each={error()}>{(item) => <li>{item.message}</li>}</For>
                </ul>
              </div>
            </Show>
          </div>
        </Dialog>
      </Show>
    </>
  );
};

export default SystemInfo;
